@@ -66,7 +66,8 @@ const char *sztid[] = {
 	"DXCC_TID",
 	"WKEY_TID",
 	"CWIO_TID",
-	"FMT_TID"
+	"FSK_TID",
+	"FMT_TID",
 	"FLMAIN_TID"
 };
 
